@@ -1,20 +1,20 @@
 /*
- * The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) implementation with additional features.
- * Copyright (C) 2017 Belledonne Communications SARL
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This file is part of oRTP.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -47,6 +47,7 @@
 #define ORTP_AVPF_FEATURE_NONE 0
 #define ORTP_AVPF_FEATURE_TMMBR (1 << 0)
 #define ORTP_AVPF_FEATURE_GENERIC_NACK (1 << 1)
+#define ORTP_AVPF_FEATURE_IMMEDIATE_NACK (1 << 2)
 
 
 typedef enum {
@@ -102,6 +103,10 @@ typedef struct _JitterControl
 	uint32_t last_log_ts;
 	uint32_t local_ts_start;
 	uint32_t remote_ts_start;
+	uint32_t diverged_start_ts;
+	bool_t is_diverging;
+	bool_t jb_size_updated;
+	bool_t pad[2];
 } JitterControl;
 
 typedef struct _WaitPoint
@@ -241,6 +246,7 @@ typedef enum {
 } OrtpRtcpXrRcvrRttMode;
 
 typedef enum {
+	OrtpRtcpXrStatSummaryNone = 0,
 	OrtpRtcpXrStatSummaryLoss = (1 << 7),
 	OrtpRtcpXrStatSummaryDup = (1 << 6),
 	OrtpRtcpXrStatSummaryJitt = (1 << 5),
@@ -301,7 +307,10 @@ typedef struct _OrtpStream {
 	struct sockaddr_storage rem_addr;
 	socklen_t loc_addrlen;
 	struct sockaddr_storage loc_addr;
+	socklen_t used_loc_addrlen;
+	struct sockaddr_storage used_loc_addr; /*Address used to redirect packets from this source*/
 	struct _RtpTransport *tr;
+	OrtpBwEstimator recv_bw_estimator;
 	struct timeval send_bw_start; /* used for bandwidth estimation */
 	struct timeval recv_bw_start; /* used for bandwidth estimation */
 	unsigned int sent_bytes; /* used for bandwidth estimation */
@@ -327,7 +336,8 @@ typedef struct _RtpStream
 	uint32_t snd_time_offset;/*the scheduler time when the application send its first timestamp*/
 	uint32_t snd_ts_offset;	/* the first application timestamp sent by the application */
 	uint32_t snd_rand_offset;	/* a random number added to the user offset to make the stream timestamp*/
-	uint32_t snd_last_ts;	/* the last stream timestamp sended */
+	uint32_t snd_last_ts;	/* the last stream timestamp sent */
+	uint16_t snd_last_nack;	/* the last nack sent when in immediate mode */
 	uint32_t rcv_time_offset; /*the scheduler time when the application ask for its first timestamp*/
 	uint32_t rcv_ts_offset;  /* the first stream timestamp */
 	uint32_t rcv_query_ts_offset;	/* the first user timestamp asked by the application */
@@ -351,6 +361,12 @@ typedef struct _RtpStream
 	int ssrc_changed_thres;
 	jitter_stats_t jitter_stats;
 	struct _OrtpCongestionDetector *congdetect;
+	struct _OrtpVideoBandwidthEstimator *video_bw_estimator;
+	ortp_thread_t win_t;
+	volatile bool_t is_win_thread_running;
+	ortp_mutex_t winthread_lock;
+	queue_t winrq;
+	ortp_mutex_t winrq_lock;
 }RtpStream;
 
 typedef struct _RtcpStream
@@ -379,6 +395,7 @@ typedef struct _RtpSession RtpSession;
 **/
 struct _RtpSession
 {
+	ortp_mutex_t main_mutex; /* To protect data that can be accessed simultaneously by a control thread and the real-time thread in charge of sending/receiving. */
 	RtpSession *next;	/* next RtpSession, when the session are enqueued by the scheduler */
 	int mask_pos;	/* the position in the scheduler mask of RtpSession : do not move this field: it is part of the ABI since the session_set macros use it*/
 	struct {
@@ -443,8 +460,28 @@ struct _RtpSession
 	bool_t use_pktinfo;
 
 	bool_t is_spliced;
+	bool_t congestion_detector_enabled;
+	bool_t video_bandwidth_estimator_enabled;
+	bool_t is_primary;  /* tells if this session is the primary of the rtp bundle */
+	
+	bool_t warn_non_working_pkt_info;
+
+	/* bundle mode */
+	struct _RtpBundle *bundle; /* back pointer to the rtp bundle object */
+	queue_t bundleq;
+	ortp_mutex_t bundleq_lock;
 };
 
+/**
+ * Structure describing the video bandwidth estimator parameters
+**/
+typedef struct _OrtpVideoBandwidthEstimatorParams {
+	int enabled; /**<Whether estimator is enabled or off.*/
+	unsigned int packet_count_min; /** minimum number of packets with the same sent timestamp to be processed continuously before being used */
+	unsigned int packets_size_max; /** number of packets needed to compute the available video bandwidth */
+	unsigned int trust_percentage; /** percentage for which the chosen bandwidth value in all available will be inferior */
+} OrtpVideoBandwidthEstimatorParams;
+
 
 
 
@@ -597,6 +634,9 @@ ORTP_PUBLIC int rtp_session_recvfrom(RtpSession *session, bool_t is_rtp, mblk_t
 ORTP_PUBLIC int rtp_session_recv_with_ts(RtpSession *session, uint8_t *buffer, int len, uint32_t ts, int *have_more);
 ORTP_PUBLIC int rtp_session_send_with_ts(RtpSession *session, const uint8_t *buffer, int len, uint32_t userts);
 
+/* Specific function called to reset the winrq queue and if called on windows to stop the async reception thread */
+ORTP_PUBLIC void rtp_session_reset_recvfrom(RtpSession *session);
+
 /* event API*/
 ORTP_PUBLIC void rtp_session_register_event_queue(RtpSession *session, OrtpEvQueue *q);
 ORTP_PUBLIC void rtp_session_unregister_event_queue(RtpSession *session, OrtpEvQueue *q);
@@ -681,6 +721,7 @@ ORTP_PUBLIC float rtp_session_get_round_trip_propagation(RtpSession *session);
 
 ORTP_PUBLIC void rtp_session_enable_network_simulation(RtpSession *session, const OrtpNetworkSimulatorParams *params);
 ORTP_PUBLIC void rtp_session_enable_congestion_detection(RtpSession *session, bool_t enabled);
+ORTP_PUBLIC void rtp_session_enable_video_bandwidth_estimator(RtpSession *session, const OrtpVideoBandwidthEstimatorParams *params);
 
 ORTP_PUBLIC void rtp_session_rtcp_set_lost_packet_value( RtpSession *session, const int value );
 ORTP_PUBLIC void rtp_session_rtcp_set_jitter_value(RtpSession *session, const unsigned int value );
@@ -720,6 +761,8 @@ ORTP_PUBLIC void rtp_session_dispatch_event(RtpSession *session, OrtpEvent *ev);
 
 ORTP_PUBLIC void rtp_session_set_reuseaddr(RtpSession *session, bool_t yes);
 
+ORTP_PUBLIC int meta_rtp_transport_sendto(RtpTransport *t, mblk_t *msg , int flags, const struct sockaddr *to, socklen_t tolen);
+
 ORTP_PUBLIC int meta_rtp_transport_modifier_inject_packet_to_send(RtpTransport *t, RtpTransportModifier *tpm, mblk_t *msg, int flags);
 ORTP_PUBLIC int meta_rtp_transport_modifier_inject_packet_to_send_to(RtpTransport *t, RtpTransportModifier *tpm, mblk_t *msg, int flags, const struct sockaddr *to, socklen_t tolen);
 ORTP_PUBLIC int meta_rtp_transport_modifier_inject_packet_to_recv(RtpTransport *t, RtpTransportModifier *tpm, mblk_t *msg, int flags);
@@ -741,12 +784,38 @@ ORTP_PUBLIC void meta_rtp_transport_set_endpoint(RtpTransport *transport,RtpTran
 
 ORTP_PUBLIC void meta_rtp_transport_destroy(RtpTransport *tp);
 ORTP_PUBLIC void meta_rtp_transport_append_modifier(RtpTransport *tp,RtpTransportModifier *tpm);
+ORTP_PUBLIC void meta_rtp_transport_prepend_modifier(RtpTransport *tp,RtpTransportModifier *tpm);
+ORTP_PUBLIC void meta_rtp_transport_remove_modifier(RtpTransport *tp, RtpTransportModifier *tpm);
 
 ORTP_PUBLIC int rtp_session_splice(RtpSession *session, RtpSession *to_session);
 ORTP_PUBLIC int rtp_session_unsplice(RtpSession *session, RtpSession *to_session);
 
 ORTP_PUBLIC bool_t ortp_stream_is_ipv6(OrtpStream *os);
 
+/* RtpBundle api */
+typedef struct _RtpBundle RtpBundle;
+
+ORTP_PUBLIC RtpBundle* rtp_bundle_new(void);
+ORTP_PUBLIC void rtp_bundle_delete(RtpBundle *bundle);
+
+ORTP_PUBLIC int rtp_bundle_get_mid_extension_id(RtpBundle *bundle);
+ORTP_PUBLIC void rtp_bundle_set_mid_extension_id(RtpBundle *bundle, int id);
+
+ORTP_PUBLIC void rtp_bundle_add_session(RtpBundle *bundle, const char *mid, RtpSession *session);
+ORTP_PUBLIC void rtp_bundle_remove_session_by_id(RtpBundle *bundle, const char *mid);
+ORTP_PUBLIC void rtp_bundle_remove_session(RtpBundle *bundle, RtpSession *session);
+ORTP_PUBLIC void rtp_bundle_clear(RtpBundle *bundle);
+
+ORTP_PUBLIC RtpSession* rtp_bundle_get_primary_session(RtpBundle *bundle);
+ORTP_PUBLIC void rtp_bundle_set_primary_session(RtpBundle *bundle, const char * mid);
+
+ORTP_PUBLIC const char *rtp_bundle_get_session_mid(RtpBundle *bundle, RtpSession *session);
+
+ORTP_PUBLIC int rtp_bundle_send_through_primary(RtpBundle *bundle, bool_t is_rtp, mblk_t *m, int flags, const struct sockaddr *destaddr, socklen_t destlen);
+/* Returns FALSE if the rtp packet or at least one of the RTCP packet (compound) was for the primary */
+ORTP_PUBLIC bool_t rtp_bundle_dispatch(RtpBundle *bundle, bool_t is_rtp, mblk_t *m);
+ORTP_PUBLIC void rtp_session_use_local_addr(RtpSession * session, const char * rtp_local_addr, const char * rtcp_local_addr);
+
 #ifdef __cplusplus
 }
 #endif
